@@ -9,11 +9,12 @@
 #                        |  |_|  /  |  |  /  |  |  / \_
 #                         \/  |_/   |_/|_/\_/|_/|_/ \/
 
-__version__ = '0.0.9'
+__version__ = '0.1.0'
 
 import re
 
 # TODO  multiple Whens for one Scenario!
+# TODO  what happens with blank table items?
 
 class Morelia:
 
@@ -25,6 +26,9 @@ class Morelia:
         self.steps = []
         self.line_number = line_number
 
+#  TODO  escape the sample regices already!
+#  and the default code should be 'print <arg_names, ... >'
+
         for s in list[::-1]:
             mpt = self.my_parent_type()
 
@@ -41,8 +45,9 @@ class Morelia:
     def my_class_name(self):  return re.sub(r'.*\.', '', str(self.__class__))
     def prefix(self):  return ''
     def my_parent_type(self):  return None
-        
-    def _my_regex(self, name):  #  TODO  calculate name inside
+
+    def _my_regex(self):  #  TODO  calculate name inside
+        name = self.i_look_like()
         return '\s*(' + name + '):?\s+(.*)'
 
     def evaluate_steps(self, v):
@@ -72,9 +77,11 @@ class Morelia:
         reconstruction = self.reconstruction().replace('\n', '\\n')
         args = (self.get_filename(), self.line_number, parent_reconstruction, reconstruction, diagnostic)
         return '\n  File "%s", line %s, in %s\n    %s\n%s' % args
-    
+   
     def reconstruction(self):
-        return self.concept + ': ' + self.predicate
+        recon = self.prefix() + self.concept + ': ' + self.predicate
+        if recon[-1] != '\n':  recon += '\n'
+        return recon
 
     def get_filename(self):
         node = self
@@ -105,7 +112,7 @@ class Viridis(Morelia):
                      '        ' + doc_string + '\n\n' + \
                      '        # code\n\n'
 
-        suite.fail(diagnostic)  #  TODO  are linefeeds tooken out of the method_name?
+        suite.fail(diagnostic)
 
 #  ERGO  river is to riparian as pond is to ___?
 
@@ -118,6 +125,8 @@ class Viridis(Morelia):
         predicate = re.sub(r'\<.+?\>', '(.+)', predicate)
         self._add_extra_args(r'"(.+?)"', predicate)
         predicate = re.sub(r'".+?"', '"([^"]+)"', predicate)
+        predicate = re.sub(r' \s+', '\\s+', predicate)
+        predicate = predicate.replace('\n', '\\n')
         return "r'" + predicate + "'"
 
     def _add_extra_args(self, matcher, predicate):
@@ -164,6 +173,7 @@ class Viridis(Morelia):
         self.find_step_name(suite)
         self.method(*self.matches)
 
+
 class Parser:  
     def __init__(self):  
         self.thangs = [ Feature, Scenario,
@@ -185,7 +195,9 @@ class Parser:
         self.rip(TestVisitor(suite))  #  CONSIDER  rename to Viridis
 
     def report(self, suite):
-        self.rip(ReportVisitor(suite))
+        rv = ReportVisitor(suite)
+        self.rip(rv)
+        return str(rv)
 
     def rip(self, v):
         if self.steps != []:
@@ -230,8 +242,7 @@ class Parser:
         
         for klass in self.thangs:
             self.thang = klass()
-            name = self.thang.i_look_like()
-            rx = self.thang._my_regex(name)
+            rx = self.thang._my_regex()
             m = re.compile(rx).match(self.line)
 
             if m and len(m.groups()) > 0:
@@ -255,9 +266,15 @@ class Parser:
 
 class ReportVisitor:
     def __init__(self, suite):  self.suite = suite
+    string = ''
 
     def visit(self, node):
-        print node.prefix() + node.reconstruction()
+        recon =  node.reconstruction()
+        if recon[-1] != '\n':  recon += '\n'  #  TODO  clean this outa def reconstruction(s)!
+        self.string += recon
+
+    def __str__(self):
+        return self.string
 
 
 class TestVisitor:
@@ -281,7 +298,7 @@ class Scenario(Morelia):
 
     def evaluate_steps(self, visitor):
         schedule = self.permute_schedule()
-        
+
         for indices in schedule:
             self.row_indices = indices
             self.evaluate_test_case(visitor)  #  note this works on reports too!
@@ -295,7 +312,8 @@ class Scenario(Morelia):
         visitor.suite.setUp()
 
         try:
-            Morelia.evaluate_steps(self, visitor)
+            visitor.visit(self)
+            for step in self.steps:  step.evaluate_steps(visitor)
         finally:
             visitor.suite.tearDown()
 
@@ -315,6 +333,9 @@ class Scenario(Morelia):
     def count_Row_dimensions(self):
         return [step.count_dimensions() for step in self.steps]
 
+    def reconstruction(self):
+        return '\n' + self.concept + ': ' + self.predicate
+
 
 class Step(Viridis):
     def my_parent_type(self):  return Scenario
@@ -322,10 +343,12 @@ class Step(Viridis):
     def evaluate_step(self, v):
         self.find_step_name(v.suite)
 
+# ERGO  use "born again pagan" somewhere
+
         try:
             self.method(*self.matches)
         except (Exception, SyntaxError), e:
-            new_exception = self.format_fault(e.args[0])
+            new_exception = self.format_fault(str(e))
             e.args = (new_exception,) + (e.args[1:])
             if type(e) == SyntaxError:  raise SyntaxError(new_exception)
             raise
@@ -372,16 +395,26 @@ class Step(Viridis):
         # CONSIDER  mix replitrons and matchers!
 
 
-class Given(Step):   pass  #  CONSIDER  distinguish these by fault signatures!
-class When(Step):   pass
-class Then(Step):  pass
-class And(Step):  pass
+class Given(Step):   #  CONSIDER  distinguish these by fault signatures!
+    def prefix(self):  return '  '
+class When(Step):  #  TODO  cycle these against the Scenario
+    def prefix(self):  return '   '
+class Then(Step):
+    def prefix(self):  return '   '
+class And(Step):  
+    def prefix(self):  return '    '
 
 #  CONSIDER  how to validate that every row you think you wrote actually ran?
 
 class Row(Morelia):
     def i_look_like(self):  return r'\|'
     def my_parent_type(self):  return Step
+    def prefix(self):  return '        '
+
+    def reconstruction(self):  #  TODO  strip the reconstruction at error time
+        recon = self.prefix() + '| ' + self.predicate
+        if recon[-1] != '\n':  recon += '\n'
+        return recon
 
     def count_dimension(self):
         if self is self.parent.steps[0]:  return 0
@@ -399,19 +432,22 @@ class Comment(Morelia):
     def i_look_like(self):  return r'\#'
     def my_parent_type(self):  return Morelia # aka "any"
 
-    def _my_regex(self, name):
+    def _my_regex(self):
+        name = self.i_look_like()
         return '\s*(' + name + ')(.*)'
 
     def validate_predicate(self):
         self.enforce(self.predicate.count('\n') == 0, 'linefeed in comment')
 
+    def reconstruction(self):
+        recon = '  # ' + self.predicate
+        if recon[-1] != '\n':  recon += '\n'
+        return recon
 
 if __name__ == '__main__':
     import os
     os.system('python ../tests/morelia_suite.py')   #  NOTE  this might not return the correct shell value
 
-#  TODO  maximum munch fails - Given must start a line
-
 
 def _special_range(n):  #  CONSIDER  better name
     return xrange(n) if n else [0]
